--
--------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 1.0
--  \   \        Filename: $RCSfile: vit_dec.vhd,v $           
--  /   /        Date Last Modified: $Date: 2008/07/02 10:58:28 $ 
-- /___/   /\    Date Created: April 2008 
-- \   \  /  \   
--  \___\/\___\
-- 
--
-------------------------------------------------------------------------------- 
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you 
-- a license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as is" solely for use in developing programs and 
-- solutions for Xilinx devices. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose. 
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
-- 
--
-- Copyright (c) 2008 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--
-- Function : circular buffer for cost and traceback training
-- with Viterbi decoder
-----------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.global_types_pkg.all;


entity vit_dec is
  port
  (
	  --inputs
    clk        : in std_logic;
	  ext2all    : in GLB_InterfaceEXT2ALL     := GLB_InterfaceEXT2ALL_DEFAULT;
	  puip       : in GLB_Data                 := GLB_Data_DEFAULT;
    cts        : in std_logic;
	  --outputs
    rfd        : out std_logic;
	  puop       : out GLB_Data;
    -- ber
    ber_done   : out std_logic;
    ber        : out std_logic_vector(15 downto 0)
   );
end vit_dec;

architecture synth of vit_dec is

constant CB_ADDR_W : INTEGER := 16; 
constant CONSTRAINT_LENGTH_M1 : integer := CONSTRAINT_LENGTH -1;
constant DATA_W : integer := SOFT_W*OUTPUT_RATE;

component vitdec_top
	port (
	data_in0: IN std_logic_VECTOR(3 downto 0);
	data_in1: IN std_logic_VECTOR(3 downto 0);
	data_in2: IN std_logic_VECTOR(3 downto 0);
	block_in: IN std_logic;
	data_out: OUT std_logic;
	ber: OUT std_logic_VECTOR(15 downto 0);
	ber_done: OUT std_logic;
	block_out: OUT std_logic;
	rdy: OUT std_logic;
	ce: IN std_logic;
	sclr: IN std_logic;
	clk: IN std_logic);
end component;


signal puip_cb,puop_cb : GLB_Data;
signal puop_dec :  GLB_Data;
-- circular buffer signals
signal cb_rfd : std_logic;
-- viterbi output
signal vit_rdy : std_logic;
signal vit_ber_done : std_logic;
signal vit_ber : std_logic_vector(15 downto 0);

begin

	------------------------------------------------------------------------------
  -- circular buffer to take input data to handle the buffering required
  -- to decode the tail biting correctly
  -- data is input sequentially and read out first training block
  -- by rest of block to be decoded followed by final training block
	------------------------------------------------------------------------------
  -- sort out input data to buffer, using puip.data to pass in sys as a vector
  -- and if we can't start a new block stop the flow
  puip_cb.valid <= puip.valid and cb_rfd;
  puip_cb.data <= puip.data;
  puip_cb.cb_start <= puip.cb_start and puip.valid and cb_rfd;
  puip_cb.cb_done <= puip.cb_done and puip.valid and cb_rfd;
  

  cb: entity work.circ_buffer_dec
    generic map
    (
      c_addr_width => CB_ADDR_W,
      c_data_width => DATA_W
    )
    port map
    (
	    -- data input handling
      data_in     => puip_cb.data(DATA_W-1 downto 0), -- data written to buffer sequentially
      fd_wr       => puip_cb.cb_start,-- first data start of buffer block -- only valid when nd and rfd are high
      ld_wr       => puip_cb.cb_done,-- last data end of buffer block -- only valid when nd and rfd are high
      nd          => puip_cb.valid,-- input data is valid if rfd is high
      rfd         => cb_rfd,--  data can be written
      -- data output handling
      cts         => cts,
      data_out    => puop_cb.data(DATA_W-1 downto 0),
      block_out   => puop_cb.valid,
      -- clocks etc
      clk         => clk,
	    ext2all     => ext2all
     );
  

	------------------------------------------------------------------------------
 	-- decode the data
  -- take the data from the circular buffer
  -- this is three blocks two training blocks with the block to be decoded in 
  -- the middle
  -- only the central block output is valid when decoded
	------------------------------------------------------------------------------

--vit7 : entity work.viterbi_v7_0 
vit7 : entity work.vitdec_top
  port map(
    -- control
    sclr     => ext2all.reset,
    ce       => ext2all.ce,
    clk      => clk,
    -- data in
    data_in0 => puop_cb.data(SOFT_W-1 downto 0),
    data_in1 => puop_cb.data(2*SOFT_W-1 downto SOFT_W),
    data_in2 => puop_cb.data(3*SOFT_W-1 downto 2*SOFT_W),
    block_in => puop_cb.valid,
    -- output
    data_out => puop_dec.data(0),
    block_out=> puop_dec.valid,
    rdy      => vit_rdy,
    -- ber
    ber_done => vit_ber_done,
    ber      => vit_ber
  );


  
	ctrl_vit: process(clk)
	begin
		if rising_edge(clk) and ext2all.ce='1' then
      if (ext2all.reset = '1') then
      else
        if (cts = '1') then
        end if;-- cts
      end if;-- reset
		end if; -- clk
	end process ctrl_vit;


	------------------------------------------------------------------------------
  -- pass out data
	------------------------------------------------------------------------------
  PUOP.data <= puop_dec.data;
  PUOP.valid <= puop_dec.valid;
  PUOP.cb_start <= puop_dec.cb_start;
  PUOP.cb_done <= puop_dec.cb_done;
  RFD <= cb_rfd;
  
  BER <= vit_ber;
  BER_DONE <= vit_ber_done;

end synth;

------------ EOF  ------------
