--------------------------------------------------------------------------------
-- Copyright (c) 2007 Xilinx, Inc.
-- All Rights Reserved
--------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   Vendor: Xilinx
-- \   \   \/    Version: 1.0
--  \   \        Filename: awgn_gen.vhd
--  /   /        Date Created: 
-- /___/   /\
-- \   \  /  \
--  \___\/\___\
--
--------------------------------------------------------------------------------
--Legal Disclaimer
--
--(c) Copyright 2002 - 2009 Xilinx, Inc. All rights reserved. 
--
--This file contains confidential and proprietary information
--of Xilinx, Inc. and is protected under U.S. and
--international copyright and other intellectual property
--1aws.
----
--DISCLAIMER
--This disclaimer is not a license and does not grant any
--rights to the materials distributed herewith. Except as
--otherwise provided in a valid license issued to you by
--Xilinx, and to the maximum extent permitted by applicable
--law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
--WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
--AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
--BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
--INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
--(2) Xilinx shall not be liable (whether in contract or tort,
--including negligence, or under any other theory of
--liability) for any loss or damage of any kind or nature
--related to, arising under or in connection with these
--materials, including for any direct, or any indirect,
--special, incidental, or consequential loss or damage
--(including loss of data, profits, goodwill, or any type of
--loss or damage suffered as a result of any action brought
--by a third party) even if such damage or loss was
--reasonably foreseeable or Xilinx had been advised of the
--possibility of the same.
----
--CRITICAL APPLICATIONS
--Xilinx products are not designed or intended to be fail-
--safe, or for use in any application requiring fail-safe
--performance, such as life-support or safety devices or
--systems, Class III medical devices, nuclear facilities,
--applications related to the deployment of airbags, or any
--other applications that could lead to death, personal
--injury, or severe property or environmental damage
--individually and collectively, "Critical
--Applications"). Customer assumes the sole risk and
--liability of any use of Xilinx products in Critical
--Applications, subject only to applicable laws and
--regulations governing limitations on product liability.
----
--THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
--PART OF THIS FILE AT ALL TIMES.--
-------------------------------------------------------------------------------
-- File       : awgn_gen.vhd
-- Company    : Xilinx
-- Last update: 2003/03/28
-- Platform   : PC
-------------------------------------------------------------------------------
-- Description: This is a AWGN genarator. It accepts the I and Q data bits
--              (QPSK case, for BPSK just ignore the Q data bit), maps it to
--              the Euclidean space, adds AWGN to it and quantize it to widthd
--              data bits. This model puts the pdf of the channel centered
--              arround + ((2**(widthd - 1)) - 1) and
--                      - ((2**(widthd - 1)) - 1)
--              The quantized output data is in two's complement format.
--              I/O description:
--              SNR    : Signal to noise ratio in dB.
--              Rate   : Code Rate.
--              i_hard : The I channel hard data bit.
--              q_hard : The Q channel hard data bit.
--              ce     : Clock enable.
--              reset  : Synchronous reset.
--              clk    : Data clock.
--              i_real : I channel noisy data before quantization.
--              q_real : Q channel noisy data before quantization.
--              i_soft : I channel quantized noisy data in 2's complement
--                       representation.
--              q_soft : Q channel quantized noisy data in 2's complement
--                       representation.
--              widthd : The number of quantized bits.
--              Note   :
--                       Variables SEED1 and SEED2 are initialized to 11 and
--                       31. By changing these vaules, different seeds of the
--                       awgn can be generated.
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2001/07/10  1.0      Xilinx
-------------------------------------------------------------------------------
LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;
USE IEEE.STD_LOGIC_UNSIGNED.ALL;
USE ieee.math_real.ALL;


LIBRARY work;
USE work.conversion_pkg.ALL;

entity awgn_gen is
  generic (
           output_rate     : INTEGER :=2;
           widthd          : integer := 3;
           SNR             : real;
           Rate            : real;
           c_has_erased    : INTEGER := 0;
           c_soft_code     : INTEGER := 0 
           );
  port( ch_hard   : in  STD_LOGIC_VECTOR(output_rate-1 DOWNTO 0);
        erased    : in  STD_LOGIC_VECTOR(output_rate -1 DOWNTO 0);
        ce        : in  STD_LOGIC;
        reset     : in  STD_LOGIC;
        clk       : in  STD_LOGIC;
        ch_soft0  : out std_logic_vector(widthd-1 downto 0);
        ch_soft1  : out std_logic_vector(widthd-1 downto 0)
        );
end awgn_gen;

architecture behav of awgn_gen is
  TYPE  ch_r IS ARRAY (0 TO 7-1) OF REAL;
  TYPE  ch_i IS ARRAY (0 TO output_rate-1) OF INTEGER;
  constant soft_signed_magnitude : integer := 0;
  constant soft_offset_binary : integer := 1;
  
  signal EbNo     : real := 1.0;
  signal Ampl     : real := 1.0;
  signal qlevel   : real := 1.0;
  signal qlevelm1 : real := 1.0;
  signal qleveln  : real := 1.0;
  signal scal     : real := 1.0;
  
  signal ch_tx     : ch_r;
  signal ch_noise  : ch_r;
  signal ch_rcv    : ch_r;
  signal ch_scaled : ch_r;

  CONSTANT del : INTEGER := 3;  
  SIGNAL ch_hard_del : STD_LOGIC_VECTOR(output_rate -1 DOWNTO 0) := (OTHERS => '0');
  TYPE  ch_array IS ARRAY (0 TO output_rate-1) OF 
                    STD_LOGIC_VECTOR(widthd+1 DOWNTO 0); 
  TYPE  ch_del_array IS ARRAY (0 TO output_rate-1) OF 
                    STD_LOGIC_VECTOR(del-1 DOWNTO 0); 
 
  SIGNAL ch_soft_int : ch_array;
  
begin

  ampgen : process (EbNo, Ampl, qlevel)
  begin
    EbNo     <= (10.0**(SNR/10.0));
    Ampl     <= SQRT(2.0*EbNo*Rate);
    if(widthd > 1) then
       qlevel <= 2.0**(widthd - 1);  
    else
       qlevel <= 2.0;
    end if;   
    qlevelm1 <= qlevel - 1.0;
    qleveln  <= -1.0*qlevelm1;
    scal     <= qlevelm1/Ampl;
  end process ampgen;

  txgen : process (clk, reset)
  variable at_start : BOOLEAN := TRUE;
  begin
   if(at_start) then
      at_start := FALSE; 
      for i in 0 to output_rate -1 LOOP 
        ch_tx(i)   <= 1.0;
      end loop;   
   end if;
    if (clk = '1' and clk'event ) then
      if (reset = '1') then
        for i in 0 to output_rate -1 LOOP 
           ch_tx(i)   <= 0.0;
        end loop;   
      elsif (ce = '1') then
        for i in 0 to output_rate -1 LOOP 
           if (ch_hard(i) = '1') then
            ch_tx(i) <= -1.0*Ampl;
           else
            ch_tx(i) <= Ampl;
           end if;
        end loop;   
      end if;
    end if;
  end process txgen;

  noisgen : process (clk, reset)
    variable SEED1 : integer := 11;
    variable SEED2 : integer := 31;
    variable U1    : real;
    variable U2    : real;
    variable at_start : BOOLEAN := TRUE;
    variable cnt : integer := 0;
  begin
   if(at_start) then
      at_start := FALSE; 
      for i in 0 to output_rate -1 LOOP 
        ch_noise(i)   <= 1.0;
      end loop;   
   end if;
    if (clk = '1' and clk'event ) then
      if (reset = '1' ) then
        for i in 0 to output_rate -1 LOOP 
         ch_noise(i) <= 0.0;
        end loop; 
      elsif (ce = '1') then
        for i in 0 to output_rate -1 LOOP 
           uniform(SEED1, SEED2, U1);
           uniform(SEED1, SEED2, U2);
           ch_noise(i) <= SQRT(-2.0*log(U1))*cos(2.0*MATH_PI*U2);
        end loop; 
        uniform(SEED1, SEED2, U1); 
        uniform(SEED1, SEED2, U2);
        ch_noise(0) <= SQRT(-2.0*log(U1))*cos(2.0*MATH_PI*U2);
        IF (output_rate >1 ) THEN
           ch_noise(1) <= SQRT(-2.0*log(U1))*sin(2.0*MATH_PI*U2);
        END IF;
      end if;
    end if;
  end process noisgen;

  rcvgen : process (clk, reset)
  variable at_start : BOOLEAN := TRUE;
  begin
   if(at_start) then
      at_start := FALSE; 
      for i in 0 to output_rate -1 LOOP 
        ch_rcv(i)   <= 1.0;
      end loop;   
   end if;
    if (clk = '1' and clk'event ) then
      if (reset = '1') then
        for i in 0 to output_rate -1 LOOP 
           ch_rcv(i) <= 0.0;
        end loop; 
      elsif (ce = '1') then
        for i in 0 to output_rate -1 LOOP 
          ch_rcv(i) <= ch_tx(i) + ch_noise(i);
        end loop; 
      end if;
    end if;
  end process rcvgen;

  scale : process (ch_rcv, scal)
  begin
    for i in 0 to output_rate -1 LOOP 
       ch_scaled(i) <= scal*ch_rcv(i);
    end loop;
  end process scale;


  quant : process (clk, reset)
    variable ch_temp : ch_r;
    variable ch_del : ch_del_array;
    variable ch_conv : ch_i;
    constant convert : INTEGER := (2**(widthd-1)+1);
    constant convert1 : INTEGER := (2**(widthd-1)-1);
    variable at_start : BOOLEAN := TRUE;
  begin
   if(at_start) then
      at_start := FALSE; 
      for i in 0 to output_rate -1 LOOP 
        ch_del(i)   := (others => '0');
        if(c_soft_code = soft_signed_magnitude ) then
          ch_soft_int(i) <= (others => '0');
        else
          ch_soft_int(i)(widthd+1 downto widthd-1) <= (others =>'0'); -- to avoid timing conflicts at start
          ch_soft_int(i)(widthd-2 downto 0) <= (others => '1');
        end if;
      end loop;   
   end if;
    if (clk = '1' and clk'event ) then
      if (reset = '1' ) then
        for i in 0 to output_rate -1 LOOP 
          ch_del(i) := (others => '0');
          ch_soft_int(i) <= (others => '0');
          if(c_soft_code = soft_signed_magnitude ) then
            ch_soft_int(i) <= (others => '0');
          else
            ch_soft_int(i)(widthd+1 downto widthd-1) <= (others =>'0'); -- to avoid timing conflicts at start
            ch_soft_int(i)(widthd-2 downto 0) <= (others => '1');
          end if;
        end loop;
      elsif (ce = '1') then
        for i in 0 to output_rate -1 LOOP 
           ch_del(i)(del-1 DOWNTO 1) := ch_del(i)(del-2 DOWNTO 0); 
           ch_del(i)(0) :=  ch_hard(i);
        
           if (ch_scaled(i) > qlevelm1) then
             ch_temp(i) := qlevelm1;
           elsif (ch_scaled(i) <= qleveln) then
             ch_temp(i) := qleveln - 1.0;
           else
             ch_temp(i) := ch_scaled(i);
           end if;
           ch_hard_del(i) <= ch_del(i)(del-1);
           -- convert integers to correct format
           ch_conv(i) := integer(floor(ch_temp(i)));
           if(widthd = 1) then
            -- hard coding
              if(ch_conv(i) < 0) then
                  ch_conv(i) := 1;
              else
                  ch_conv(i) := 0;
              end if; 
           else
              if( c_soft_code = soft_signed_magnitude ) then
                 if(ch_conv(i) < 0) then
                    ch_conv(i) := -ch_conv(i) - convert;
                 end if;       
              else
                 if(ch_conv(i) < 0) then
                    ch_conv(i) := -ch_conv(i) - convert;
                 end if; 
                 if(ch_conv(i) >= 0) then
                    ch_conv(i) := -ch_conv(i) + convert1;
                 end if;       
              end if;
           end if; 
           IF(c_has_erased /= 0) THEN 
              IF(erased(i) = '1') THEN
               ch_conv(i) := 0;
              END IF;
           END IF;
           ch_soft_int(i) <= to_vector(widthd+2, ch_conv(i));
        end loop;   
      end if;
    end if;
  end process quant;

  ch_soft0 <= ch_soft_int(0)(widthd-1 downto 0);
  ch_soft1 <= ch_soft_int(1)(widthd-1 downto 0);
end behav;
